% =====================================================================
% ====    This program solves the two-country
% ====    heterogeneous agent-incomplete markets model in
% ====    "A generalized time iteration method for solving dynamic 
% ====    optimization problems with occasionally binding constraints"
% ====    by Ayse Kabukcuoglu-Dur and Enrique Martinez-Garcia
% ====    Date: March 2020
% =====================================================================
clear all; clc; 


 
%----------------------------------------------------------------
% 1. Parameterization
%---------------------------------------------------------------- 
global  prob delta delta2 beta alpha Nstar SSprob M ag eps Ic ap1 ap2 A1 A2 psi1 psi2 B gamma eta 

%=====Stochastic process ==================================================
Nstar = 3;                 % number of states for the efficiency shock.
Z = [.167;0.839;5.087];    % Effective labor shock values. 

pi11=.9; pi22=.99; pi33=.9;                  
prob   = [pi11, 1-pi11, 0; (1-pi22)/2, pi22, (1-pi22)/2; 0, 1-pi11, pi11];
                                             %Markov transition prob matrix
SSprob = prob^500;  %steady state probability distribution

%===== Production and utility function=====================================

alpha  = 0.36;             % Share of capital in production
beta   = 0.971;            % Subjective discount factor         
gamma=2;                   % Utility function parameters: c^(1-gamma)/(1-gamma) - B n^(1+1/eta)/(1+1/eta)
eta=.5;                    % Frisch elasticity
B=60;                      % Labor disutility parameter
delta  = .06;              % Depreciation rate
delta2 = .06;                              
A1 =.6;                    % TFP parameters
A2= 1;                                         

%====== Government policy parameters ======================================

tauk1 =.397;               % Capital income tax in country1
tauk2 = .425;              % Capital income tax in country2 
taun1 =  1-.27;            % Labor income tax in country1 
taun2 = 1-.33;             % Labor income tax in country2 
psi1 = 1-0.151;            % Labor income tax progressivity parameter for country 1 (defined as 1-psi in the paper)
psi2 = 1-0.151;            % Labor income tax progressivity parameter for country 2
DY1 = 0.70;                % Public debt/GDP in country 1           
DY2= 0.937;                % Public debt/GDP in country 2

%===== Borrowing limits ===================================================

abar1 =0; abar2 =0;    

%===== Grid points (next period assets ====================================

amin1 = abar1-2;           % Minimum value of the asset grid - Country 1
amin2 = abar2-2;           % Minimum value of the asset grid - Country 2 
amax1 = 200;               % Maximum value of grid (common)             

N=6000;                    % Number of asset grid points for the policy function
M=N*20;                    % Number of asset grid points for the ergodic distribution
ap1=linspace(amin1,amax1,N)';
ap2 = ap1;
ag=linspace(amin1,amax1,M); %Distribution function grid points

eps =.00001; %min consumption level


%----------------------------------------------------------------
% 2. Solve the model
%----------------------------------------------------------------  


%===International interest rate============================================

rmax=  .028; %or use 1/beta-1;
rmin= .0249; 

r=.5*rmax+.5*rmin;

%==========================================================================

% Some useful stuff

n = ones(1,Nstar); 
T=12000; % Number of periods in the simulation
T0 =2000; %Number of initial periods omitted in the simulation
     


Gamma1 = ones(Nstar,1)*max(ag/ag(M),0); Gamma1=Gamma1';  %Initializing the ergodic distribution
Gamma2=Gamma1;

a1=zeros(N,Nstar); a2=zeros(N,Nstar);     % Initializing current assets
cp1 = (1+r)*r*ap1*n;                      % Initial guess for consumption policy function      
cp2 = (1+r)*r*ap2*n;  
Ic = 400;                            


s = 0;            % Initial value of the counter (needed for the bisection method used
                  % to find the equilibrium interest rate (see Aiyagari (1994))).
time1=cputime;

MarketClearing=1;  damp=0.51; %damping parameter (set =0.5 for standard bisection, or increase up to 1 to slow down convergence)

while abs(MarketClearing)>1e-6  
   
s = s+1;                               % s is the # of iterations started.
rk1 = delta+(r/(1-tauk1)); 
kn1 = (rk1/(A1*alpha))^(1/(alpha-1));  % Implied K/N ratio
w1 = (1-alpha)*A1*kn1^(alpha);         % Wage implied by the capital labor ratio.

[cp1,c1,ainv1,np1]=sub_hh_problem1(cp1,ap1,taun1,r,w1,abar1,Z,1); %Solving the household optimization problem to find c'(a'), c(a'), a(a'), n'(a')

[Gamma1,bin1]=sub_ergodicdist(Gamma1,ap1,ainv1,ag);    % Stationary cumulative distribution function

g1 = [Gamma1(1,:);Gamma1(2:M,:)-Gamma1(1:M-1,:)]; %prob. density function

av = [ag(1);((ag(1:M-1)+ag(2:M))/2)']; %grid points for the distribution functions 
                                       % since they are finer than asset grid points, 
                                       % we use interpolation in the next step
cv1 =zeros(M,Nstar); tl1 =zeros(M,Nstar);  
  for j=1:Nstar
     cv1(:,j) = interp1(ap1,cp1(:,j),av(:,1),'linear'); %consumption policy at finer grid points
  end

 for j=1:Nstar
     for i=1:M
     nv1(i,j)=find_labor2(cv1(i,j),Z(j),w1,psi1,taun1); %labor policy at finer grid points
     end
 end
meanA1 = sum(sum(g1*(SSprob(1,:)').*av)); %Total assets
meanN1 = sum(sum(SSprob(1,:)*Z*(g1'.*nv1'))); % Effective labor
meanK1=kn1*meanN1; %Capital 
meanY1=A1*(meanK1^alpha)*(meanN1^(1-alpha)); % Output
D1=DY1*meanY1;  % Public debt
B1=meanA1-meanK1-D1; %Net foreign assets 

%Now repeating these steps for country 2:

rk2 = delta2+(r/(1-tauk2));
kn2 = (rk2/(A2*alpha))^(1/(alpha-1));
w2 = (1-alpha)*A2*kn2^(alpha); 
[cp2,c2,ainv2,np2]=sub_hh_problem1(cp2,ap2,taun2,r,w2,abar2,Z,2);
[Gamma2,bin2]=sub_ergodicdist(Gamma2,ap2,ainv2,ag);

g2 = [Gamma2(1,:);Gamma2(2:M,:)-Gamma2(1:M-1,:)]; 
av = [ag(1);((ag(1:M-1)+ag(2:M))/2)'];

cv2=zeros(M,Nstar);tl2 =zeros(M,Nstar);
for j=1:Nstar
 cv2(:,j) = interp1(ap2,cp2(:,j),av(:,1),'linear');
 end 

 for j=1:Nstar
      for i=1:M
     
      nv2(i,j)=find_labor2(cv2(i,j),Z(j),w2,psi2,taun2);
     
      end
  end

 meanN2 = sum(sum(SSprob(1,:)*Z*(g2'.*nv2')));
 meanA2 = sum(sum(g2*(SSprob(1,:)').*av)); 
 meanK2=kn2*meanN2;
 meanY2=A2*(meanK2^alpha)*(meanN2^(1-alpha));
 D2=DY2*meanY2;
 B2=meanA2-meanK2-D2;
 
 
MarketClearing = (B1+B2);

if MarketClearing>0 
    rmax=r;
    r=rmin*(1-damp)+r*damp;
    
else 
    rmin=r;
    r=rmax*(1-damp)+r*damp;
    
end
    disp(['interest rate bracket' '  ' '& market clearing'])
    disp([rmin*100 rmax*100 MarketClearing])
end

time2=cputime;
cpuseconds=time2-time1;

disp('CPU time(s):');
disp(cpuseconds);

%Other government policy variables
for j=1:Nstar
     for i=1:M
     tl1(i,j)= nv1(i,j)*w1*Z(j,1)-sub_ptax2(psi1,taun1,nv1(i,j)*w1*Z(j,1)); %tax on labor income
     tl2(i,j)= nv2(i,j)*w2*Z(j,1)-sub_ptax2(psi2,taun2,nv2(i,j)*w2*Z(j,1)); %tax on labor income
     end
end

meanT1 = sum(sum(g1*(SSprob(1,:)').*tl1));  meanT2 = sum(sum(g2*(SSprob(2,:)').*tl2)); %Tax revenues on labor income
G1=(meanK1*(rk1-delta)*tauk1+meanT1-r*D1); G2=(meanK2*(rk2-delta)*tauk2+meanT2-r*D1);
GY1=G1/meanY1;  GY2=G2/meanY2; %Government spending-to-GDP ratio


%----------------------------------------------------------------
% 3. Evaluate accuracy
%----------------------------------------------------------------  
Zt = MarkovChain(prob,T);

%========== Country 1=======================================
[meanAS1,at1,ct1] = HouseholdSim(cp1,ap1,abar1,ainv1,Zt,meanA1) ;
[intertemporal_errors]=sub_foc_errors(ainv1,ap1,cp1,r,abar1,Z);
 
for t=1:T-T0
sim_intertemp_ee(t)=interp1(ap1(ap1>=abar1),intertemporal_errors(ap1>=abar1,Zt(t)),at1(t),'pchip')*(at1(t+1)>abar1);
end
sim_intertemp_ee1=sim_intertemp_ee((sim_intertemp_ee~=0));
intertemporal_errors1=intertemporal_errors;
maxEE1=max(log10(abs(sim_intertemp_ee1))); meanEE1=mean(log10(abs(sim_intertemp_ee1)));

%=========== Country 2 ========================================
 [meanAS2,at2,ct2] = HouseholdSim(cp2,ap2,abar2,ainv2,Zt,meanA2) ;
 [intertemporal_errors]=sub_foc_errors(ainv2,ap2,cp2,r,abar2,Z);

 
 for t=1:T-T0;
 sim_intertemp_ee2(t)=interp1(ap2(ap2>=abar2),intertemporal_errors(ap2>=abar2,Zt(t)),at2(t),'pchip')*(at2(t+1)>abar2);
 end
sim_intertemp_ee2=sim_intertemp_ee2((sim_intertemp_ee2~=0));
intertemporal_errors2=intertemporal_errors;
maxEE2=max(log10(abs(sim_intertemp_ee2))); meanEE2=mean(log10(abs(sim_intertemp_ee2)));

%----------------------------------------------------------------
% 4. Plot policy functions and Euler equation errors
%---------------------------------------------------------------- 
for j=1:Nstar
a_pol1(:,j)=interp1(ainv1(ap1>=abar1,j),ap1(ap1>=abar1),ap1,'linear','extrap'); 
a_pol2(:,j)=interp1(ainv2(ap2>=abar2,j),ap2(ap2>=abar2),ap2,'linear','extrap');
end

a_pol1=max(abar1,a_pol1); a_pol2=max(abar2,a_pol2);

figure(1);
subplot(3,2,1)
plot(ap1,a_pol1);
xlim([0 20])
%ylim([0  20])
title('Decision rules for assets in Country 1');

subplot(3,2,2)
plot(ap2,a_pol2);
xlim([0 20])
%ylim([0  20])
title('Decision rules for assets in Country 2');

subplot(3,2,3)
plot(ap1,cp1);
xlim([0 20])
%ylim([0  6])
title('Decision rules for consumption in Country 1');

subplot(3,2,4)
plot(ap2,cp2);
xlim([0 20])
%ylim([0  6])
title('Decision rules for consumption in Country 2');

subplot(3,2,5)
plot(ap1,np1);
xlim([0 20])
%ylim([0  1])
title('Decision rules for labor supply in Country 1');

subplot(3,2,6)
plot(ap2,np2);
xlim([0 20])
%ylim([0  1])
title('Decision rules for labor supply in Country 2');

figure;
subplot(1,2,1)
EEE1 = log10(abs(intertemporal_errors1));
EEE1(ap1<abar1,:)=NaN;
%plot(ap1,EEE1);
plot(ap1,EEE1);
xlim([0 amax1]); ylim([-7 -1]);
title('Euler equation errors - Country 1');


subplot(1,2,2)
EEE2 = log10(abs(intertemporal_errors2));
EEE2(ap2<abar2,:)=NaN;
%plot(ap1,EEE1);
plot(ap2,EEE2);
xlim([0 amax1]); ylim([-7 -1]);
title('Euler equation errors - Country 2 ');


 

%-------------------------------------------------------------------------
% 5. Gini coefficients and Lorenz curves
%--------------------------------------------------------------------------

% ainv0{1}=ainv1; ainv0{2}=ainv2;
% wealth11= zeros(Nstar*M,1);gammadist11= zeros(Nstar*M,1);
% wealth22= zeros(Nstar*M,1);gammadist22= zeros(Nstar*M,1);
% 
% for j=1:Nstar
%     a1 = (cv1(av>=abar1,j)+av(av>=abar1,1)-sub_ptax2(psi1,taun1,nv1(1,j)*w1*Z(j,1)))./(1+r); 
%     
%     ainv1(av>=abar1,j) = a1;
%     ainv1(av<abar1,j)=min(av(1,1),a1(1)); 
%      a2 = (cv2(av>=abar2,j)+av(av>=abar2,1)-sub_ptax2(psi2,taun2,nv2(1,j)*w2*Z(j,1)))./(1+r); 
%      
%      ainv2(av>=abar2,j) = a2;
%      ainv2(av<abar2,j)=min(av(1,1),a2(1)); 
% end
%  av1=av(av>=0); Mm=length(av);
% 
%  
%  
% wealth1=vertcat(av(av>=abar1,1),av(av>=abar1,1),av(av>=abar1,1)); 
% wealth2=vertcat(av(av>=abar2,1),av(av>=abar2,1),av(av>=abar2,1));
% 
% 
% 
% gammadist1=vertcat(SSprob(1,1)*g1(av>=abar1,1),SSprob(1,2)*g1(av>=abar1,2),SSprob(1,3)*g1(av>=abar1,3));
% gammadist2=vertcat(SSprob(1,1)*g2(av>=abar2,1),SSprob(1,2)*g2(av>=abar2,2),SSprob(1,3)*g2(av>=abar2,3));
% 
% 
% [wealthgini1,wealthlorenz1,walorenz1] = sub_gini(gammadist1,wealth1,false);
% [wealthgini2,wealthlorenz2,walorenz2] = sub_gini(gammadist2,wealth2,false);